<?php

/* --------------------------------------------------------------
   ContentMetaKeywords.inc.php 2019-04-12
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ContentMetaKeywords
 *
 * This class represents the localized content meta keywords
 *
 * @category   System
 * @package    Content
 */
class ContentMetaKeywords implements LocalizedContentAttributeInterface
{
    /**
     * Content meta keywords
     *
     * @var StringType
     */
    protected $metaKeywords;
    
    /**
     * Language code
     *
     * @var string
     */
    protected $languageCode;
    
    
    /**
     * ContentMetaKeywords constructor
     *
     * @param \StringType $metaKeywords Content meta keywords
     * @param LanguageCode $languageCode Language code
     */
    public function __construct(StringType $metaKeywords, LanguageCode $languageCode)
    {
        if (!strlen($metaKeywords->asString())) {
            throw new InvalidArgumentException('Empty meta keywords provided');
        }
        
        $this->metaKeywords = $metaKeywords;
        $this->languageCode = $languageCode->asString();
    }
    
    
    /**
     * Return the content meta keywords
     *
     * @return string
     */
    public function content()
    {
        return $this->metaKeywords->asString();
    }
    
    
    /**
     * Return the language code
     *
     * @return string
     */
    public function languageCode()
    {
        return $this->languageCode;
    }
}