<?php

/* --------------------------------------------------------------
   ContentText.inc.php 2019-04-11
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ContentText
 *
 * This class represents the localized content text
 *
 * @category   System
 * @package    Content
 */
class ContentText implements LocalizedContentAttributeInterface
{
    /**
     * Content text
     *
     * @var StringType
     */
    protected $text;
    
    /**
     * Language code
     *
     * @var string
     */
    protected $languageCode;
    
    
    /**
     * ContentText constructor
     *
     * @param \StringType $text Content text
     * @param LanguageCode $languageCode Language code
     */
    public function __construct(StringType $text, LanguageCode $languageCode)
    {
        if (!strlen($text->asString())) {
            throw new InvalidArgumentException('Empty text provided');
        }
        
        $this->text = $text;
        $this->languageCode = $languageCode->asString();
    }
    
    
    /**
     * Return the content text
     *
     * @return string
     */
    public function content()
    {
        return $this->text->asString();
    }
    
    
    /**
     * Return the language code
     *
     * @return string
     */
    public function languageCode()
    {
        return $this->languageCode;
    }
}