<?php
/* --------------------------------------------------------------
   dynamic_theme_style.css.php 2019-04-04
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/*
 * DYNAMIC THEME STYLE 
 * 
 * This script will handle the creation and delivery of the requested theme CSS.
 * 
 * OPTIONAL GET PARAMETERS 
 * 
 * - style_name: the currently active StyleEdit3 style name. 
 * - style_edit_cache: toggles the use of the StyleEdit3 CSS cache file.   
 * - stop_style_edit: disables style edit style loading from being included in the generated content.   
 * - renew_cache: enforces the style cache renewal. 
 * 
 */

error_reporting(E_ALL & ~E_NOTICE & ~E_DEPRECATED & ~E_STRICT & ~E_CORE_ERROR & ~E_CORE_WARNING);

@date_default_timezone_set('Europe/Berlin');

// Setup style required constants.
define('CACHE_FILENAME', '__dynamics.css');
define('CACHE_FILENAME_STYLE_EDIT', '__dynamics_style_edit.css');
define('DIR_PUBLIC_THEME', 'public/theme/');

// Setup optional parameter aliases.
$styleName      = isset($_GET['style_name']) ? $_GET['style_name'] : null;
$styleEditCache = isset($_GET['style_edit_cache']) ? $_GET['style_edit_cache'] : null;
$renewCache     = isset($_GET['renew_cache']) ? $_GET['renew_cache'] : null;

function setMemoryLimit($limitInMegaBytes = 256)
{
	$minMemoryLimitGiven = false;
	$minMemoryLimit      = (string)$limitInMegaBytes . 'M';
	
	if(function_exists('ini_get') && function_exists('ini_set'))
	{
		$serverMemoryLimit = @ini_get('memory_limit');
		
		if(preg_match('/[\d]+M/', $serverMemoryLimit))
		{
			$memoryLimit = (int)substr($serverMemoryLimit, 0, -1);
			if($memoryLimit < $limitInMegaBytes)
			{
				@ini_set('memory_limit', $minMemoryLimit);
				if(@ini_get('memory_limit') === $minMemoryLimit)
				{
					$minMemoryLimitGiven = true;
				}
			}
			else
			{
				$minMemoryLimitGiven = true;
			}
		}
		elseif(preg_match('/^[\d]+$/', $serverMemoryLimit))
		{
			$memoryLimit    = (int)$serverMemoryLimit;
			$minMemoryLimit = $limitInMegaBytes * 1024 * 1024;
			
			if($memoryLimit < $minMemoryLimit)
			{
				@ini_set('memory_limit', $minMemoryLimit);
				if(@ini_get('memory_limit') === $minMemoryLimit)
				{
					$minMemoryLimitGiven = true;
				}
			}
			else
			{
				$minMemoryLimitGiven = true;
			}
		}
	}
	
	return $minMemoryLimitGiven;
}

setMemoryLimit();

header('Content-Type: text/css; charset=utf-8');

$scriptError = '';

function errorHandler($errorNumber, $errorMessage, $errorFile, $errorLine)
{
	if(!(error_reporting() & $errorNumber))
	{
		return null; // This error code is not included in error_reporting.
	}
	
	$GLOBALS['scriptError'] .= str_replace('"', '\"', "[CSS compile error] '$errorMessage' in $errorFile:$errorLine ");
	
	echo '
		body:before {
			content: "' . $GLOBALS['scriptError'] . '";
			position: absolute;
			background: #FF5722;
			color: #fff;
			top: 0;
			left: 0;
			z-index: 100000;
			padding: 15px;
		}
	';
	
	return true; // Don't execute PHP internal error handler.
}

set_error_handler('errorHandler');

// Include necessary system files for GXModulesCache integration.
require_once file_exists('includes/local/configure.php') ? 'includes/local/configure.php' : 'includes/configure.php';
require_once 'system/core/logging/LogControl.inc.php';
require_once 'system/core/caching/DataCache.inc.php';
require_once 'system/core/caching/GXModulesCache.inc.php';
require_once 'system/core/GoogleFontDownloader.inc.php';
require_once 'vendor/natxet/CssMin/src/CssMin.php';
require_once 'GXMainComponents/Extensions/ScssCompiler/ScssCompilerFactory.php';

// Fetch theme name from theme configuration file.
$themeConfigurationFilePath = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'theme.json';
$themeConfiguration         = json_decode(file_get_contents($themeConfigurationFilePath), true);
$themeId                    = $themeConfiguration['id'];

$suffix = '.min';
if(file_exists(DIR_FS_CATALOG . '.dev-environment'))
{
	$suffix = '';
}

$additionalCssFiles  = [];
$additionalScssFiles = [];
$additionalScssPaths = [];

if(file_exists(DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/stylesheet.css'))
{
	$additionalCssFiles[] = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/stylesheet.css';
}

if(file_exists(DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/vendor' . $suffix . '.css'))
{
	$additionalCssFiles[] = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/vendor' . $suffix . '.css';
}

if(file_exists(DIR_FS_CATALOG . 'JSEngine/build/vendor' . $suffix . '.css'))
{
	$additionalCssFiles[] = DIR_FS_CATALOG . 'JSEngine/build/vendor' . $suffix . '.css';
}

if(file_exists(DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/vendor_fixes.css'))
{
	$additionalCssFiles[] = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/vendor_fixes.css';
}

$gxModulesFiles = GXModulesCache::getInstalledModuleFiles();
foreach($gxModulesFiles as $file)
{
	if(stripos($file, '/Themes/' . $themeId . '/Css/') !== false
	   || stripos($file, '/Themes/All/Css/') !== false)
	{
		if(substr($file, -4) === '.css')
		{
			$additionalCssFiles[] = $file;
		}
		elseif(substr($file, -9) === 'main.scss')
		{
			$additionalScssFiles[] = $file;
			$additionalScssPaths[] = substr(substr($file, 0, strripos($file, '/')), strlen(DIR_FS_CATALOG));
		}
	}
}
$additionalScssFiles = array_unique($additionalScssFiles);
$additionalScssPaths = array_unique($additionalScssPaths);
$cacheFilePath       = DIR_FS_CATALOG . 'cache/' . ($styleEditCache
                                                    === null ? CACHE_FILENAME : CACHE_FILENAME_STYLE_EDIT);

if($styleName !== null
   && file_exists('StyleEdit3/bootstrap.inc.php')
   && file_exists(DIR_PUBLIC_THEME . 'styles/styleedit'))
{
	try
	{
		include_once 'StyleEdit3/bootstrap.inc.php';
		
		if(\StyleEdit\Authentication::isAuthenticated())
		{
			$cacheFilePath = DIR_FS_CATALOG . 'cache/__dynamics-' . md5($styleName) . '.css';
		}
	}
	catch(\Exception $exception)
	{
		// Continue without StyleEdit3.
	}
}

$createCache = false;

if($renewCache !== null || !file_exists($cacheFilePath) || filesize($cacheFilePath) < 10)
{
	$createCache = true;
}

function getFileModificationTime($p_file)
{
	$lastModified = filemtime($p_file);
	
	// Windows time fix
	if(date('I', $lastModified) != 1 && date('I') == 1)
	{
		$lastModified += 3600;
	}
	elseif(date('I', $lastModified) == 1 && date('I') != 1)
	{
		$lastModified -= 3600;
	}
	
	return $lastModified;
}

function minifyCss($css, array $filters, array $plugins)
{
	$cssMinifier = new CssMinifier($css, $filters, $plugins);
	
	return $cssMinifier->getMinified();
}

if($createCache === false)
{
	ob_start();
	
	foreach($additionalCssFiles as $additionalCssFile)
	{
		include $additionalCssFile;
		
		// Add comment to close unclosed comment in included file.
		echo "\n/**/\n";
	}
	
	$additionalCss = ob_get_clean();
	
	if($suffix === '.min')
	{
		$filters = [
			'ImportImports'                 => false,
			'RemoveComments'                => true,
			'RemoveEmptyRulesets'           => true,
			'RemoveEmptyAtBlocks'           => true,
			'ConvertLevel3AtKeyframes'      => false,
			'ConvertLevel3Properties'       => false,
			'Variables'                     => true,
			'RemoveLastDelarationSemiColon' => true
		];
		
		$plugins = [
			'Variables'                => true,
			'ConvertFontWeight'        => true,
			'ConvertHslColors'         => true,
			'ConvertRgbColors'         => true,
			'ConvertNamedColors'       => true,
			'CompressColorValues'      => true,
			'CompressUnitValues'       => true,
			'CompressExpressionValues' => true
		];
		
		$additionalCss = minifyCss($additionalCss, $filters, $plugins);
	}
	
	echo file_get_contents($cacheFilePath) . PHP_EOL . $additionalCss;
}
else
{
	$css = '';

	$fileToServe = 'main.scss';
	$basePath    = DIR_PUBLIC_THEME . 'styles/system/';  // Default theme directory.
	$pathPattern = DIR_FS_CATALOG . 'cache/*.css*';
	$cachePaths  = glob($pathPattern);
	
	if(is_array($cachePaths))
	{
		foreach($cachePaths as $result)
		{
			if(strpos($result, '__dynamic') === false)
			{
				unlink($result);
			}
		}
	}
	
	if(file_exists($cacheFilePath))
	{
		@unlink($cacheFilePath);
	}
	
	$errorOccurred         = false;
	$errorMessage          = '';
	$styleEditErrorMessage = '';
	
	if(file_exists('StyleEdit3/bootstrap.inc.php')
	   && file_exists(DIR_PUBLIC_THEME . 'styles/styleedit'))
	{
		try
		{
			include_once 'StyleEdit3/bootstrap.inc.php';
		
			if(class_exists('\StyleEdit\StyleConfigReader'))
			{
				$_REQUEST['theme']     = 'true'; // Toggle theme mode for StyleEdit.
				$styleConfigReader     = \StyleEdit\StyleConfigReader::getInstance($themeId, $styleName);
				$bootstrapScss         = $styleConfigReader->getScss('bootstrap', $styleName);
				$templateScss          = $styleConfigReader->getScss('template', $styleName);
				$customStylesScss      = $styleConfigReader->getCustomStyles($styleName);
				
				// Download google fonts and use the local fonts
				$fontUrl = $styleConfigReader->findSettingValueByName('gx-font-import-url');
				if(strpos($fontUrl, '"') === 0 || strpos($fontUrl, '\'') === 0)
				{
					$fontUrl = substr($fontUrl, 1, strlen($fontUrl) - 2);
				}
				
				if($fontUrl !== null &&
					(strpos($fontUrl, 'https://fonts.googleapis.com') !== false
				       || strpos($fontUrl, 'http://fonts.googleapis.com') !== false))
				{
					$fsLocalFont = DIR_FS_CATALOG . 'public/fonts/' . md5($fontUrl) . '.css';
		
					if($request_type === 'SSL' || !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
					{
						define('GM_HTTP_SERVER', HTTPS_SERVER);
					}
					else
					{
						define('GM_HTTP_SERVER', HTTP_SERVER);
					}
		
					$wsLocalFont = GM_HTTP_SERVER . DIR_WS_CATALOG . 'public/fonts/' . md5($fontUrl) . '.css'; // HTTP_SERVER . DIR_WS_CATALOG
					
					if(!file_exists($fsLocalFont))
					{
						$fontDownloader = new GoogleFontDownloader();
						$fontDownloader->downloadFont($fontUrl);
					} // No else if, just in case the download failed.
					
					if(file_exists($fsLocalFont))
					{
						$templateScss .= "\n\n" . '$gx-font-import-url: "";';
						$templateScss .= "\n" . file_get_contents($fsLocalFont);
					}
				}
				
				if($styleConfigReader->getErrorMessage() !== '')
				{
					$styleEditErrorMessage = 'body:before {
						content: "' . str_replace('"', '\"', $styleConfigReader->getErrorMessage()) . '";
						position: absolute;
						background: white;
						top: 0;
						left: 0;
						z-index: 100000;
					}';
				}
				
				$customDirectoryPath = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system/custom/';
				
				if(file_exists($customDirectoryPath . '_usermod.scss'))
				{
					$customStylesScss .= "\n\n@import \"custom/usermod\";";
				}
				
				if(@file_put_contents($customDirectoryPath . '_bootstrap_variables.scss', $bootstrapScss) === false)
				{
					$errorOccurred = true;
				}
				
				if(@file_put_contents($customDirectoryPath . '_template_variables.scss', $templateScss) === false)
				{
					$errorOccurred = true;
				}
				
				if(@file_put_contents($customDirectoryPath . '_custom_styles.scss', $customStylesScss) === false)
				{
					$errorOccurred = true;
				}
				
				if($errorOccurred)
				{
					$errorMessage = 'body:before {
										content: "The directory ' . $customDirectoryPath . ' and/or containing files are not writable. CSS cache file cannot be created causing slow page speed.";
										position: absolute;
										background: white;
										top: 0;
										left: 0;
										z-index: 100000;
									}';
					
					$css .= $errorMessage;
				}
			}
			else
			{
				throw new \RuntimeException('Class \'StyleEdit\StyleConfigReader\' not found');
			}
		}
		catch(\Exception $e)
		{
			$errorOccurred        = true;
			$errorMessage = '
		        body:before {
					content: "' . str_replace('"', '\"', $e->getMessage()) . '";
					position: absolute;
					background: white;
					top: 0;
					left: 0;
					z-index: 100000;
				}
			';
		
			$css .= $errorMessage;
		}
	}
	else
	{
		$customDirectoryPath = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/custom/';
		
		if(!file_exists($customDirectoryPath . '_bootstrap_variables.scss'))
		{
			@file_put_contents($customDirectoryPath . '_bootstrap_variables.scss', '');
		}
		
		if(!file_exists($customDirectoryPath . '_template_variables.scss'))
		{
			@file_put_contents($customDirectoryPath . '_template_variables.scss', '');
		}
		
		if(!file_exists($customDirectoryPath . '_custom_styles.scss'))
		{
			$customStylesScss = '';
			
			if(file_exists($customDirectoryPath . '_usermod.scss'))
			{
				$customStylesScss = "\n\n@import \"custom/usermod\";";
			}
			
			@file_put_contents($customDirectoryPath . '_custom_styles.scss', $customStylesScss);
		}
		else
		{
			$customStylesScss = file_get_contents($customDirectoryPath . '_custom_styles.scss');
			
			if(file_exists($customDirectoryPath . '_usermod.scss'))
			{
				if(strpos($customStylesScss, '@import "custom/usermod";') === false)
				{
					$customStylesScss .= "\n\n@import \"custom/usermod\";";
					
					@file_put_contents($customDirectoryPath . '_custom_styles.scss', $customStylesScss);
				}
			}
			elseif(strpos($customStylesScss, '@import "custom/usermod";') !== false)
			{
				$customStylesScss = str_replace('@import "custom/usermod";', '', $customStylesScss);
				
				@file_put_contents($customDirectoryPath . '_custom_styles.scss', $customStylesScss);
			}
		}
	}
	
	$compiler = ScssCompilerFactory::create()->createCompiler();
	
	// Serve CSS
	$compiler->setVariables([
		'theme-base-path' => '"' . DIR_WS_CATALOG . DIR_PUBLIC_THEME . '"'
	]);
	
	$compiler->addImportPath($basePath);
	$compiler->setBasePath(DIR_FS_CATALOG . $basePath);
	
	if(count($additionalScssPaths) > 0)
	{
		foreach($additionalScssPaths as $additionalScssPath)
		{
			$compiler->addImportPath($additionalScssPath);
		}
	}
	
	if($suffix === '.min')
	{
		$compiler->setFormatter(ScssCompilerInterface::STYLE_COMPRESSED);
	}
	else
	{
		$compiler->setFormatter(ScssCompilerInterface::STYLE_NESTED);
	}
	
	$compiler->setAdditionalScssFiles($additionalScssFiles);
	
	ob_start();
	try
	{
		$compiler->serve($fileToServe);
	}
	catch(\Exception $exception)
	{
		$errorMessage = '
			body:before {
				content: "' . str_replace('"', '\"', $exception->getMessage()) . '";
				position: absolute;
				background: white;
				top: 0;
				left: 0;
				z-index: 100000;
			}
		';
		
		echo $errorMessage . $styleEditErrorMessage;
		
		if(file_exists(DIR_FS_CATALOG . 'cache/' . CACHE_FILENAME))
		{
			echo file_get_contents(DIR_FS_CATALOG . 'cache/' . CACHE_FILENAME);
		}
		
		$errorOccurred = true;
	}
	
	$scss = ob_get_clean();
	
	if(strpos($scss, 'Parse error') === 0)
	{
		$errorMessage = 'body:before {
							content: "' . str_replace('"', '\"', str_replace("\n", ' ', str_replace("\r", '', $scss))) . '";
							position: absolute;
							background: white;
							top: 0;
							left: 0;
							z-index: 100000;
						}';
		
		$scss = $errorMessage . $styleEditErrorMessage;
		
		if(file_exists(DIR_FS_CATALOG . 'cache/' . CACHE_FILENAME))
		{
			$scss .= file_get_contents(DIR_FS_CATALOG . 'cache/' . CACHE_FILENAME);
		}
		
		$errorOccurred = true;
	}
	
	// Compile default css in error case.
	if($errorOccurred)
	{
		ob_start();
		
		try
		{
			$compiler = ScssCompilerFactory::create()->createCompiler();
			$compiler->setVariables([
				'theme-base-path' => '"' . DIR_WS_CATALOG . DIR_PUBLIC_THEME . '"'
			]);
			$compiler->addImportPath($basePath);
			$compiler->setBasePath(DIR_FS_CATALOG . $basePath);
			if(count($additionalScssPaths) > 0)
			{
				foreach($additionalScssPaths as $additionalScssPath)
				{
					$compiler->addImportPath($additionalScssPath);
				}
			}
			
			if($suffix === '.min')
			{
				$compiler->setFormatter(ScssCompilerInterface::STYLE_COMPRESSED);
			}
			else
			{
				$compiler->setFormatter(ScssCompilerInterface::STYLE_EXPANDED);
			}
			
			$compiler->setAdditionalScssFiles($additionalScssFiles);
			
			// delete custom scss forcing to compile default css to ensure a working frontend
			@file_put_contents($customDirectoryPath . '_bootstrap_variables.scss', '');
			@file_put_contents($customDirectoryPath . '_template_variables.scss', '');
			@file_put_contents($customDirectoryPath . '_custom_styles.scss', '');
			
			$compiler->serve($fileToServe);
			
			$scss = ob_get_clean();
			$scss = $errorMessage . "\n" . $styleEditErrorMessage . "\n" . $scss;
		}
		catch(\Exception $e)
		{
			ob_clean();
		}
	}
	
	$scss = str_replace('\"', '"', $scss);
	$css  .= $scss;
	
	if(!$errorOccurred && $scriptError === '')
	{
		if($suffix === '.min')
		{
			$css = preg_replace('!/\*.*?\*/!s', '', $css);
		}
		
		file_put_contents($cacheFilePath, $css);
	}
	
	header('HTTP/1.1 200 OK');
	header('Last-Modified: ' . gmdate('r'), true);
	header('ETag: ', true);
	header('Cache-Control: no-cache, must-revalidate', true);
	
	ob_start();
	
	foreach($additionalCssFiles as $additionalCssFile)
	{
		include $additionalCssFile;
		
		// print comment to close unclosed comment in included file
		echo "\n/**/\n";
	}
	
	$additionalCss = ob_get_clean();
	
	if($suffix === '.min')
	{
		$filters = [
			'ImportImports'                 => false,
			'RemoveComments'                => true,
			'RemoveEmptyRulesets'           => true,
			'RemoveEmptyAtBlocks'           => true,
			'ConvertLevel3AtKeyframes'      => false,
			'ConvertLevel3Properties'       => false,
			'Variables'                     => true,
			'RemoveLastDelarationSemiColon' => true
		];
		
		$plugins = [
			'Variables'                => true,
			'ConvertFontWeight'        => true,
			'ConvertHslColors'         => true,
			'ConvertRgbColors'         => true,
			'ConvertNamedColors'       => true,
			'CompressColorValues'      => true,
			'CompressUnitValues'       => true,
			'CompressExpressionValues' => true
		];
		
		$additionalCss = minifyCss($additionalCss, $filters, $plugins);
	}
	
	$destinationDirectoryPath = DIR_FS_CATALOG . DIR_PUBLIC_THEME . 'styles/system';
	
	if($styleName === null && is_writable($destinationDirectoryPath))
	{
		$mainCssFilePath = $destinationDirectoryPath . '/main' . $suffix . '.css';
		
		if(file_exists($mainCssFilePath))
		{
			unlink($mainCssFilePath);
		}
		
		file_put_contents($mainCssFilePath, $css . PHP_EOL . $additionalCss);
	}
	
	echo $css . PHP_EOL . $additionalCss;
}
